@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli.h,v 1.6 2003/02/15 09:11:38 evertonm Exp $
+  $Id: ruli.h,v 1.8 2004/05/28 22:17:03 evertonm Exp $
   */
 
 
@@ -30,6 +30,7 @@ Boston, MA 02111-1307, USA.
 #include <ruli_mem.h>
 #include <ruli_util.h>
 #include <ruli_list.h>
+#include <ruli_addr.h>
 #include <ruli_sock.h>
 #include <ruli_txt.h>
 #include <ruli_res.h>
@@ -38,6 +39,7 @@ Boston, MA 02111-1307, USA.
 #include <ruli_search.h>
 #include <ruli_smtp.h>
 #include <ruli_sync.h>
+#include <ruli_getaddrinfo.h>
 
 
 #endif /* RULI_H */
