@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_parse.c,v 1.17 2004/06/01 18:37:18 evertonm Exp $
+  $Id: ruli_parse.c,v 1.18 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -31,6 +31,7 @@ Boston, MA 02111-1307, USA.
 #include <ruli_parse.h>
 #include <ruli_mem.h>
 #include <ruli_txt.h>
+#include <ruli_res.h>
 
 
 int ruli_parse_new(ruli_parse_t *parse)
@@ -486,4 +487,43 @@ int ruli_parse_rr_mx(ruli_mx_rdata_t *mx_rdata,
   return RULI_PARSE_RR_OK;
 }
 		     
+void ruli_parse_addr_rr(ruli_addr_t *addr, const ruli_rr_t *rr, long options)
+{
+  assert(ruli_rr_type_is_address(options, rr->type));
+
+  switch (rr->type) {
+  case RULI_RR_TYPE_A:
+    {
+      int result = ruli_parse_rr_a(&addr->addr.ipv4, 
+				   rr->rdata, rr->rdlength);
+      assert(!result); /* IN A parsing can't fail */
+    }
+    addr->addr_family = PF_INET;
+    addr->addr_size = sizeof(addr->addr.ipv4);
+    break;
+    
+  case RULI_RR_TYPE_AAAA:
+    {
+      int result = ruli_parse_rr_aaaa(&addr->addr.ipv6, 
+				      rr->rdata, rr->rdlength);
+      assert(!result); /* IN AAAA parsing can't fail */
+    }
+    addr->addr_family = PF_INET6;
+    addr->addr_size = sizeof(addr->addr.ipv6);
+    break;
+    
+  default:
+    /* Previous ruli_rr_type_is_address() call ensures
+     * RR type is either IN_A or IN_AAAA, thus we can
+     * panic otherwise.
+     */
+    assert(0);
+  }
+
+#ifdef RULI_RES_DEBUG
+  fprintf(stderr, "DEBUG: ruli_parse_addr_rr(): addr=");
+  ruli_addr_print(stderr, addr);
+  fprintf(stderr, "\n");
+#endif
+}
 
