@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.c,v 1.29 2004/05/28 01:16:49 evertonm Exp $
+  $Id: ruli_res.c,v 1.30 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -129,6 +129,18 @@ const char *ruli_get_qclass_name(int qc_code)
   return 0;
 }
 
+int ruli_rr_type_is_address(long options, int rr_type)
+{
+  switch (rr_type) {
+  case RULI_RR_TYPE_A:
+    return !(options & RULI_RES_OPT_SRV_NOINET);
+  case RULI_RR_TYPE_AAAA:
+    return !(options & RULI_RES_OPT_SRV_NOINET6);
+  }
+
+  return 0;
+}
+
 const char *ruli_res_errstr(int result)
 {
   return "FIXME: ruli_res_errstr()";
