@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.h,v 1.19 2004/06/01 18:37:30 evertonm Exp $
+  $Id: ruli_res.h,v 1.20 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -116,8 +116,10 @@ enum {
  * Query options
  */
 enum {
-  RULI_RES_OPT_VOID   = 0,
-  RULI_RES_OPT_SEARCH = 1 << 0 /* Use search list */
+  RULI_RES_OPT_VOID        = 0,
+  RULI_RES_OPT_SEARCH      = 1 << 0, /* Use search list */
+  RULI_RES_OPT_SRV_NOINET  = 1 << 1, /* Don't search IPv4 addresses */
+  RULI_RES_OPT_SRV_NOINET6 = 1 << 2  /* Don't search IPv6 addresses */
 };
 
 typedef struct {
@@ -252,6 +254,7 @@ int ruli_get_qtype_code(const char *qt_name);
 int ruli_get_qclass_code(const char *qc_name);
 const char *ruli_get_qtype_name(int qt_code);
 const char *ruli_get_qclass_name(int qc_code);
+int ruli_rr_type_is_address(long options, int rr_type);
 
 const char *ruli_res_errstr(int result);
 
