@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.58 2004/06/06 04:27:48 evertonm Exp $
+  $Id: ruli_srv.c,v 1.61 2004/06/09 17:57:15 evertonm Exp $
   */
 
 
@@ -64,6 +64,9 @@ static void *query_done(ruli_srv_t *srv_qry, int srv_result_code)
   return _ruli_srv_query_done(srv_qry, srv_result_code);
 }
 
+/*
+  dispose fallback query
+ */
 static void f_query_done(ruli_srv_t *srv_qry) {
   assert(srv_qry->fall_query);
   
@@ -72,6 +75,9 @@ static void f_query_done(ruli_srv_t *srv_qry) {
   srv_qry->fall_query = 0;
 }
 
+/*
+  finish fallback query
+ */
 static void *fall_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 {
 #ifdef RULI_SRV_DEBUG
@@ -85,6 +91,9 @@ static void *fall_query_done(ruli_srv_t *srv_qry, int srv_result_code)
   return query_done(srv_qry, srv_result_code);
 }
 
+/*
+  dispose walk query
+ */
 static void w_query_done(ruli_srv_t *srv_qry)
 {
   assert(srv_qry->walk_query);
@@ -94,6 +103,9 @@ static void w_query_done(ruli_srv_t *srv_qry)
   srv_qry->walk_query = 0;
 }
 
+/*
+  finish walk query
+ */
 static void *walk_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 {
 #ifdef RULI_SRV_DEBUG
@@ -109,6 +121,22 @@ static void *walk_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 
 static void *srv_answer_walk(ruli_srv_t *srv_qry);
 
+static int need_query_in_a(int qtype, long options)
+{
+  return ( (qtype == RULI_RR_TYPE_AAAA) &&
+	   !(options & RULI_RES_OPT_SRV_NOINET) );
+}
+
+static int reset_qtype(long options)
+{
+  /* can't disable all address families */
+  assert( !((options & RULI_RES_OPT_SRV_NOINET) &&
+	    (options & RULI_RES_OPT_SRV_NOINET6)) );
+
+  return (options & RULI_RES_OPT_SRV_NOINET6) ? 
+    RULI_RR_TYPE_A : RULI_RR_TYPE_AAAA;
+}
+
 static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 {
   ruli_srv_t       *srv_qry = (ruli_srv_t *) qry_arg;
@@ -129,8 +157,6 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
   assert(addr_list);
 
   prev_addr_list_size = ruli_list_size(addr_list);
-  assert(!prev_addr_list_size);
-
 
 #ifdef RULI_SRV_DEBUG
   {
@@ -170,8 +196,20 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
     srv_qry->w_query_rcode = rcode;
 
-    if (rcode != RULI_RCODE_NOERROR)
+    if (rcode != RULI_RCODE_NOERROR) {
+
+      if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+	/* Dispose auxiliary query as it will be re-created
+	   by next walk query */
+	w_query_done(srv_qry);
+
+	srv_qry->walk_qtype = RULI_RR_TYPE_A;
+
+	return srv_answer_walk(srv_qry); /* repeat walk index */
+      }
+
       return walk_query_done(srv_qry, RULI_SRV_CODE_WALK_RCODE);
+    }
   }
 
   /*
@@ -282,26 +320,80 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	assert(!result);
 	
 	fprintf(stderr, 
-		"DEBUG: on_walk_answer(): query target=%s target_len=%d: "
+		"DEBUG: on_walk_answer(): target=%s len=%d w_qtype=%d w_index=%d: "
 		"BUT: no matching address record\n", 
-		target_txt, target_txt_len);
+		target_txt, target_txt_len, srv_qry->walk_qtype, srv_qry->walk_index);
       }
 #endif
 
+      if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+	/* Dispose auxiliary query as it will be re-created
+	   by next walk query */
+	w_query_done(srv_qry);
+
+	srv_qry->walk_qtype = RULI_RR_TYPE_A;
+
+	return srv_answer_walk(srv_qry); /* repeat walk index */
+      }
+
       return walk_query_done(srv_qry, RULI_SRV_CODE_WALK_EMPTY);
     }
   }
 
-  /* In future walk query, consider next answer target */
-  ++srv_qry->walk_index;
+  if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+    /* Dispose auxiliary query as it will be re-created
+       by next walk query */
+    w_query_done(srv_qry);
+
+    srv_qry->walk_qtype = RULI_RR_TYPE_A;
+
+    return srv_answer_walk(srv_qry); /* repeat walk index */
+  }
 
   /* Dispose auxiliary query as it will be re-created
      by next walk query */
   w_query_done(srv_qry);
 
+  /* For the next target, use the first address type */
+  srv_qry->walk_qtype = reset_qtype(srv_qry->srv_options);
+
+  /* In future walk query, consider next answer target */
+  ++srv_qry->walk_index;
+
   return srv_answer_walk(srv_qry);
 }
 
+static int find_addr_type(const ruli_list_t *addr_list, int qtype)
+{
+  int list_size = ruli_list_size(addr_list);
+  int i;
+
+  switch (qtype) {
+  case RULI_RR_TYPE_A:
+
+    for (i = 0; i < list_size; ++i) {
+      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      if (ruli_addr_family(addr) == PF_INET)
+	return -1;
+    }
+    break;
+
+  case RULI_RR_TYPE_AAAA:
+
+    for (i = 0; i < list_size; ++i) {
+      ruli_addr_t *addr = ruli_list_get(addr_list, i);
+      if (ruli_addr_family(addr) == PF_INET6)
+	return -1;
+    }
+    break;
+
+  default:
+    assert(0);
+  }
+
+  return 0;
+}
+
 /*
   Search missing addresses for targets.
 
@@ -314,16 +406,15 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
   int         srv_list_size = ruli_list_size(srv_list);
 
   /*
-   * Scan answer targets, considering empty address lists
+   * Scan answer targets, considering address lists
    */
   for (; srv_qry->walk_index < srv_list_size; ++srv_qry->walk_index) {
     ruli_srv_entry_t *entry = (ruli_srv_entry_t *) ruli_list_get(srv_list, 
 						   srv_qry->walk_index);
-    ruli_list_t      *addr_list     = &entry->addr_list;
-    int              addr_list_size = ruli_list_size(addr_list);
+    ruli_list_t *addr_list = &entry->addr_list;
 
     /* If this target already has address(es), skip it */
-    if (addr_list_size > 0)
+    if (find_addr_type(addr_list, srv_qry->walk_qtype))
       continue;
 
 #ifdef RULI_SRV_DEBUG
@@ -338,8 +429,8 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
       assert(!result);
       
       fprintf(stderr, 
-	      "DEBUG: srv_answer_walk(): missing target=%s\n", 
-	      target_txt);
+	      "DEBUG: srv_answer_walk(): missing target=%s walk_index=%d walk_qtype=%d\n", 
+	      target_txt, srv_qry->walk_index, srv_qry->walk_qtype);
     }
 #endif
     
@@ -351,6 +442,9 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
     if (!srv_qry->walk_query)
       return query_done(srv_qry, RULI_SRV_CODE_WALK_OTHER);
 
+    assert( (srv_qry->walk_qtype == RULI_RR_TYPE_A) ||
+	    (srv_qry->walk_qtype == RULI_RR_TYPE_AAAA) );
+
     /*
      * Initialize walk query arguments
      */
@@ -359,11 +453,9 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
     srv_qry->walk_query->q_domain        = entry->target;
     srv_qry->walk_query->q_domain_len    = entry->target_len;
     srv_qry->walk_query->q_class         = RULI_RR_CLASS_IN;
+    srv_qry->walk_query->q_type          = srv_qry->walk_qtype;
     srv_qry->walk_query->q_options       = RULI_RES_OPT_VOID;
 
-    /* FIXME: perform 2 queries instead? (IN_A, IN_AAAA) */
-    srv_qry->walk_query->q_type          = RULI_RR_TYPE_ANY;
-    
     /*
      * Submit walk query
      */
@@ -385,11 +477,34 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
   return query_done(srv_qry, RULI_SRV_CODE_OK);
 }
 
+static void *fallback_resubmit(ruli_srv_t *srv_qry)
+{
+  /* reset fallback query, but not dispose it */
+  assert(srv_qry->fall_query);
+  ruli_res_query_delete(srv_qry->fall_query);
+
+  srv_qry->fall_query->q_type = srv_qry->fall_qtype;
+
+  assert(!need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options));
+
+  if (ruli_res_query_submit(srv_qry->srv_resolver, srv_qry->fall_query)) {
+    ruli_free(srv_qry->fall_query);
+    srv_qry->fall_query = 0;
+    return query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
+  }
+
+  return OOP_CONTINUE;
+}
+
 static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 {
   ruli_srv_t *srv_qry           = (ruli_srv_t *) qry_arg;
   int        prev_srv_list_size = ruli_list_size(&srv_qry->answer_srv_list);
 
+  /* fallback query adds 1 srv entry at most */
+  assert(prev_srv_list_size >= 0);
+  assert(prev_srv_list_size <= 1);
+
 #ifdef RULI_SRV_DEBUG
   {
     char txt_dname_buf[RULI_LIMIT_DNAME_TEXT_BUFSZ];
@@ -403,8 +518,8 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     assert(!result);
 
     fprintf(stderr, 
-	    "DEBUG: on_fallback_answer(): domain=%s domain_len=%d\n",
-	    txt_dname_buf, txt_dname_len);
+	    "DEBUG: on_fallback_answer(): domain=%s len=%d f_qtype=%d\n",
+	    txt_dname_buf, txt_dname_len, srv_qry->fall_qtype);
 
     fprintf(stderr, 
 	    "DEBUG: on_fallback_answer(): id=%d answer_code=%d rcode=%d\n",
@@ -415,7 +530,6 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
   assert(qry->answer_code != RULI_SRV_CODE_VOID);
   assert(srv_qry->fall_query);
   assert(srv_qry->fall_query == qry);
-  assert(!prev_srv_list_size);
 
   /*
    * Query failed?
@@ -437,8 +551,15 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 
     srv_qry->f_query_rcode = rcode;
 
-    if (rcode != RULI_RCODE_NOERROR)
+    if (rcode != RULI_RCODE_NOERROR) {
+
+      if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+	srv_qry->fall_qtype = RULI_RR_TYPE_A;
+	return fallback_resubmit(srv_qry);
+      }
+
       return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_RCODE);
+    }
   }
 
   /*
@@ -491,7 +612,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     for (;;) {
       ruli_list_t      *an_list     = &parse.answer_list;
       int              an_list_size = ruli_list_size(an_list);
-      ruli_srv_entry_t *srv_entry   = 0;
+      ruli_srv_entry_t *srv_entry   = prev_srv_list_size ? ruli_list_top(&srv_qry->answer_srv_list) : 0;
       int              i;
 
 #ifdef RULI_SRV_DEBUG
@@ -510,8 +631,8 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	
 	fprintf(stderr, 
 		"DEBUG: on_fallback_answer(): "
-		"wanted owner=(%d)%s\n", 
-		wanted_txt_len, wanted_txt);
+		"wanted owner=(%d)%s answer_list_len=%d\n", 
+		wanted_txt_len, wanted_txt, an_list_size);
       }
 #endif
 
@@ -607,11 +728,17 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
       {
 	int curr_srv_list_size = ruli_list_size(&srv_qry->answer_srv_list);
 	
-	assert(curr_srv_list_size >= prev_srv_list_size);
-	assert(curr_srv_list_size <= (prev_srv_list_size + 1));
+	assert(curr_srv_list_size >= 0);
+	assert(curr_srv_list_size <= 1);
 	
-	if (curr_srv_list_size > prev_srv_list_size) {
+	if (curr_srv_list_size > 0) {
 	  ruli_parse_delete(&parse);
+
+	  if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+	    srv_qry->fall_qtype = RULI_RR_TYPE_A;
+	    return fallback_resubmit(srv_qry);
+	  }
+
 	  return fall_query_done(srv_qry, RULI_SRV_CODE_OK);
 	}
       }
@@ -718,6 +845,11 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     
   } /* parsed answer context */
   
+  if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+    srv_qry->fall_qtype = RULI_RR_TYPE_A;
+    return fallback_resubmit(srv_qry);
+  }
+
   return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_EMPTY);
 }
 
@@ -744,7 +876,7 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
 #endif
 
   assert(!srv_qry->fall_query);
-  assert(!ruli_list_size(&srv_qry->answer_srv_list));
+  assert(srv_qry->fall_qtype == -1);
 
   /*
    * Allocate space for fallback query
@@ -753,6 +885,8 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
   if (!srv_qry->fall_query)
     return query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
 
+  srv_qry->fall_qtype = reset_qtype(srv_qry->srv_options);
+
   /*
    * Initialize fallback query arguments
    */
@@ -761,11 +895,9 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
   srv_qry->fall_query->q_domain        = srv_qry->srv_domain;
   srv_qry->fall_query->q_domain_len    = srv_qry->srv_domain_len;
   srv_qry->fall_query->q_class         = RULI_RR_CLASS_IN;
+  srv_qry->fall_query->q_type          = srv_qry->fall_qtype;
   srv_qry->fall_query->q_options       = srv_qry->srv_options;
 
-  /* FIXME: perform 2 queries instead? (IN_A, IN_AAAA) */
-  srv_qry->fall_query->q_type          = RULI_RR_TYPE_ANY;
-
   /*
    * Submit fallback query
    */
@@ -1296,7 +1428,10 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
   assert(!srv_qry->walk_query);
   assert(srv_qry->walk_index == -1);
+  assert(srv_qry->walk_qtype == -1);
+
   srv_qry->walk_index = 0;
+  srv_qry->walk_qtype = reset_qtype(srv_qry->srv_options);
 
   return srv_answer_walk(srv_qry);
 }
@@ -1312,6 +1447,10 @@ int _ruli_srv_query_submit(ruli_srv_t *srv_qry,
   assert(srv_qry);
   assert(fallback_call);
 
+  /* can't disable all address families */
+  assert( !((srv_qry->srv_options & RULI_RES_OPT_SRV_NOINET) &&
+	    (srv_qry->srv_options & RULI_RES_OPT_SRV_NOINET6)) );
+
   srv_qry->fallback = fallback_call;
 
   /*
@@ -1404,8 +1543,10 @@ int _ruli_srv_query_submit(ruli_srv_t *srv_qry,
   srv_qry->answer_code = RULI_SRV_CODE_VOID;
 
   srv_qry->walk_index = -1;
+  srv_qry->walk_qtype = -1;
   srv_qry->walk_query = 0;
 
+  srv_qry->fall_qtype = -1;
   srv_qry->fall_query = 0;
 
   /*
