@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.h,v 1.29 2004/05/24 02:43:21 evertonm Exp $
+  $Id: ruli_srv.h,v 1.31 2004/06/09 00:18:21 evertonm Exp $
   */
 
 
@@ -113,10 +113,12 @@ struct ruli_srv_t {
 
   int              walk_index;     /* index for answer_srv_list */
   ruli_res_query_t *walk_query;    /* auxiliary query context for "walk" */
-  int              w_query_rcode;
+  int              walk_qtype;     /* current walk qtype (IN_A, IN_AAAA) */
+  int              w_query_rcode;  /* rcode for walk query */
 
   ruli_res_query_t *fall_query;    /* auxiliary query context for fallback */
-  int              f_query_rcode;
+  int              fall_qtype;     /* current fall qtype (IN_A, IN_AAAA) */
+  int              f_query_rcode;  /* rcode for fall query */
 
   void *(*fallback)(ruli_srv_t *self); /* fallback function */
 
